<?php

namespace Drp\JsonApi\Exceptions;

use Drp\JsonApi\Http\Concerns\RespondsWithJsonApi;
use Illuminate\Auth\Access\AuthorizationException;
use Illuminate\Auth\AuthenticationException;
use Illuminate\Database\Eloquent\ModelNotFoundException;
use Illuminate\Http\Request;
use Illuminate\Http\Response;
use Illuminate\Validation\ValidationException;
use Symfony\Component\HttpFoundation\Response as SymfonyResponse;
use Symfony\Component\HttpKernel\Exception\HttpException;

class JsonApiHandler
{
    use RespondsWithJsonApi;

    public function render(Request $request, \Throwable $exception): ?SymfonyResponse
    {
        if ($this->isApiCall($request) === false) {
            return null;
        }

        if ($exception instanceof JsonApiException) {
            return $this->respondWithError(
                $exception->getHttpStatusCode(),
                $exception->getMessage(),
                $exception->getHint()
            );
        }

        if ($exception instanceof HttpException) {
            return $this->respondWithError(
                $exception->getStatusCode(),
                (new \ReflectionClass($exception))->getShortName(),
                $exception->getMessage()
            );
        }

        if ($exception instanceof AuthenticationException) {
            return $this->respondWithError(
                Response::HTTP_FORBIDDEN,
                'Forbidden',
                'You are not authorized to perform this action'
            );
        }

        if ($exception instanceof AuthorizationException) {
            return $this->respondWithError(
                Response::HTTP_FORBIDDEN,
                'Forbidden',
                'You are not authorized to perform this action'
            );
        }

        if ($exception instanceof ValidationException) {
            return $this->respondWithValidationError($exception->errors());
        }

        if ($exception instanceof ModelNotFoundException) {
            return $this->respondWithError(
                Response::HTTP_NOT_FOUND,
                'Not Found',
                'The URI requested is invalid or the resource requested does not exist'
            );
        }

        return null;
    }
}
